# Copyright (c) Jupyter Development Team.
# Distributed under the terms of the Modified BSD License.

import os
import sys
from setuptools import setup

# Get location of this file at runtime
HERE = os.path.abspath(os.path.dirname(__file__))

# Eval the version tuple and string from the source
VERSION_NS = {}
with open(os.path.join(HERE, 'jupyter_cms', '_version.py')) as f:
    exec(f.read(), {}, VERSION_NS)

install_requires=[
    'ipython>=4.1.0',
    'nbconvert>=5.0.0',
    'notebook>=4.2.0',
    'whoosh>=2.7.0,<3.0',
]

# Use the built-in version of scandir if possible,
# otherwise require the scandir module
try:
    from os import scandir
except ImportError:
    install_requires.append('scandir>=1.1,<2.0')

setup_args = dict(
    name='jupyter_cms',
    author='Jupyter Development Team',
    author_email='jupyter@googlegroups.com',
    description='Extension for Jupyter Notebook 4.0.x with experimental content management features',
    long_description='''
    This package adds the following features to Jupyter Notebook:

* Search dialog on file tree, editor, and notebook screens to search over filenames and .ipynb content in the notebook directory
* IPython kernel extension to make notebooks importable, and notebook cells injectable via # <api> and # <help> annotations
* Full-page drag-and-drop upload target
* Pop-over table of contents navigation for notebooks

See `the project README <https://github.com/jupyter-incubator/contentmanagement>`_
for more information.
''',
    url='https://github.com/jupyter-incubator/contentmanagement',
    version=VERSION_NS['__version__'],
    license='BSD',
    platforms=['Jupyter Notebook 4.0.x', 'Jupyter Notebook 4.1.x', 'Jupyter Notebook 4.2.x'],
    packages=[
        'jupyter_cms'
    ],
    include_package_data=True,
    scripts=[
        'scripts/jupyter-cms',
    ],
    install_requires=install_requires,
    classifiers=[
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5'
    ]
)

if 'setuptools' in sys.modules:
    # setupstools turns entrypoint scripts into executables on windows
    setup_args['entry_points'] = {
        'console_scripts': [
            'jupyter-cms = jupyter_cms.extensionapp:main',
        ]
    }
    # Don't bother installing the .py scripts if if we're using entrypoints
    setup_args.pop('scripts', None)

if __name__ == '__main__':
    setup(**setup_args)



# Copyright (c) Jupyter Development Team.
# Distributed under the terms of the Modified BSD License.

import os
import unittest
import jupyter_cms.loader as loader
from os.path import join as pjoin
from os.path import abspath, dirname

def spy():
    '''Really lightweight spy for tracking a single function call.'''
    def spy(value):
        if spy.call is not None:
            raise RuntimeError('spy already called')
        spy.call = value
    spy.call = None
    return spy

class TestNotebookPathFinder(unittest.TestCase):
    '''Tests for traversing paths within the loader root directory.'''
    def setUp(self):
        self.root = pjoin(dirname(abspath(__file__)), 'resources')
        loader.enable(self.root)

    def tearDown(self):
        loader.disable()

    def test_mywb_module(self):
        '''Should import the root as an empty module.'''
        import mywb as mod
        self.assertEqual(mod.__name__, 'mywb')
        self.assertEqual(mod.__package__, mod.__name__)
        self.assertEqual(mod.__path__, [self.root])

    def test_mywb_subdir_submodule(self):
        '''Should import subdirectories as empty packages.'''
        import mywb.subdir as mod
        self.assertEqual(mod.__name__, 'mywb.subdir')
        self.assertEqual(mod.__package__, mod.__name__)
        # print(dir(mod.__path__))
        # self.assertEqual(str(mod.__path__), [os.path.join(self.root, 'subdir')])

        import mywb.subdir.subsubdir as mod
        self.assertEqual(mod.__name__, 'mywb.subdir.subsubdir')
        self.assertEqual(mod.__package__, mod.__name__)
        # assert mod.__path__ == [os.path.join(self.root, 'subdir', 'subdir2')]

    def test_mywb_subdir_not_found(self):
        '''Should fail to import non-existent subdirectories.'''
        try:
            import mywb.subdir2
        except ImportError:
            pass
        else:
            self.assertTrue(False, 'expected ImportError')

        try:
            import mywb.fake
        except ImportError:
            pass
        else:
            self.assertTrue(False, 'expected ImportError')

    def test_normal_packages(self):
        '''Should import normal packages as before.'''
        import xml
        import xml.dom
        import xml.dom.minidom

class TestNotebookFinder(unittest.TestCase):
    '''Tests for traversing notebooks within the loader root directory.'''
    def setUp(self):
        self.root = pjoin(dirname(abspath(__file__)), 'resources')
        loader.enable(self.root)

    def tearDown(self):
        loader.disable()

    def test_mywb_notebook(self):
        '''Should import a notebook as a module.'''
        import mywb.test_injectable as mod
        self.assertEqual(mod.__name__, 'mywb.test_injectable')
        self.assertEqual(mod.__package__, 'mywb')

    def test_mywb_subdir_notebook(self):
        '''Should import a notebook in a subdir as a module.'''
        import mywb.subdir.test_injectable as mod
        self.assertEqual(mod.__name__, 'mywb.subdir.test_injectable')
        self.assertEqual(mod.__package__, 'mywb.subdir')

    def test_notebook_not_found(self):
        '''Should fail to import non-existent notebooks.'''
        try:
            import kawb.not_help
        except ImportError:
            pass
        else:
            self.assertTrue(False, 'expected ImportError')

        try:
            import mywb.not_help
        except ImportError:
            pass
        else:
            self.assertTrue(False, 'expected ImportError')

    def test_general_not_found(self):
        '''Should fail import non-existent modules as before.'''
        try:
            import not_found
        except ImportError:
            pass
        else:
            self.assertTrue(False, 'expected ImportError')
            
    def test_cwd_notebook(self):
        '''Should not import a notebook in the cwd.'''
        # switch to resources so that notebook are in the cwd
        path = os.getcwd()
        os.chdir(self.root)        
        try:
            import test_injectable as mod
        except ImportError as e:
            pass
        else:
            self.assertTrue(False, 'expected ImportError')
        finally:
            os.chdir(path)
            

class TestNotebookLoader(unittest.TestCase):
    '''Tests for loading notebooks within the loader root directory.'''
    def setUp(self):
        self.root = pjoin(dirname(abspath(__file__)), 'resources')
        loader.enable(self.root)

    def tearDown(self):
        loader.disable()

    def test_mywb_notebook(self):
        '''Should import a notebook as a module with a top level help().'''
        import mywb.test_injectable as mod

        self.assertEqual(mod.__name__, 'mywb.test_injectable')
        self.assertEqual(mod.__file__, os.path.join(self.root, 'test_injectable.ipynb'))
        self.assertEqual(mod.__package__, 'mywb')

        self.assertTrue(hasattr(mod, 'help'))
        self.assertTrue(callable(mod.help))

    def test_mywb_empty_notebook(self):
        '''Should import an empty notebook as an empty module.'''
        import mywb.empty as mod

        self.assertEqual(mod.__name__, 'mywb.empty')
        self.assertEqual(mod.__file__, os.path.join(self.root, 'empty.ipynb'))
        self.assertEqual(mod.__package__, 'mywb')

    def test_mywb_subdir_notebook(self):
        '''Should import a notebook in a subdir as a module with a top level help().'''
        import mywb.subdir.test_injectable as mod

        self.assertEqual(mod.__name__, 'mywb.subdir.test_injectable')
        self.assertEqual(mod.__file__, os.path.join(self.root, 'subdir/test_injectable.ipynb'))
        self.assertEqual(mod.__package__, 'mywb.subdir')

        self.assertTrue(hasattr(mod, 'help'))
        self.assertTrue(callable(mod.help))

    def test_load_by_path(self):
        '''Should import a notebook as a module using a function instead of import.'''
        for path in [
            os.path.join(self.root, 'test_injectable.ipynb'),
            os.path.join(self.root, 'subdir', 'test_injectable.ipynb')
        ]:
            mod = loader.load_notebook(path)
            self.assertTrue(mod.__name__)
            self.assertEqual(mod.__file__, path)
            self.assertEqual(mod.__package__, None)

class TestNotebookExposure(unittest.TestCase):
    '''Tests for APIs exposed by loaded notebooks.'''
    def setUp(self):
        self.root = pjoin(dirname(abspath(__file__)), 'resources')
        loader.enable(self.root)

        # purpose driven mocking
        loader.display = lambda x: None
        self.render = loader.HTML = spy()

    def tearDown(self):
        loader.disable()

    def test_module_help(self):
        '''Should return rich help.'''
        import mywb.test_injectable as mod
        self.assertTrue(callable(mod.help))
        self.assertEqual(mod.help.__doc__, loader.MODULE_DOCSTRING_TMPL)
        self.assertTrue(mod.__doc__.endswith(loader.MODULE_API_DOCSTRING_TMPL))
        mod.help()
        self.assertIn('Fake Module!', self.render.call)
        self.assertNotIn('help', self.render.call)

    def test_api(self):
        '''Should expose a notebook function as a module function.'''
        import mywb.test_injectable as mod
        self.assertTrue(callable(mod.some_func))
        self.assertIn('existing docstring', mod.some_func.__doc__)
        self.assertTrue(mod.some_func.__doc__.endswith(loader.API_DOCSTRING_TMPL.format(name='some_func')))
        self.assertEqual(mod.some_func(), 'fake-return')

    def test_api_help(self):
        '''Should expose a help method on a notebook function.'''
        import mywb.test_injectable as mod
        self.assertTrue(callable(mod.some_func.help))
        self.assertEqual(mod.some_func.help.__doc__, loader.BOUND_HELP_DOCSTRING_TMPL)
        mod.some_func.help()
        self.assertIn('A function!', self.render.call)
        self.assertIn('some_func', self.render.call)
        self.assertIn('fake-return', self.render.call)
        self.assertNotIn('help', self.render.call)

    def test_class_api_help(self):
        '''Should expose a help method on a notebook class.'''
        import mywb.test_injectable as mod
        self.assertTrue(callable(mod.SomeClass.help))
        mod.SomeClass.help()
        self.assertIn('SomeClass', self.render.call)
        self.assertIn('some_method', self.render.call)
        self.assertIn('True', self.render.call)

    def test_recipe_help(self):
        '''Should expose cell contents as a help recipe.'''
        import mywb.test_injectable as mod
        self.assertTrue(callable(mod.some_recipe))
        self.assertFalse(hasattr(mod.some_recipe, 'help'))
        self.assertEqual(mod.some_recipe.__doc__, loader.UNBOUND_HELP_DOCSTRING_TMPL.format(name='some_recipe'))
        mod.some_recipe()
        self.assertIn('A Recipe!', self.render.call)
        self.assertIn('print', self.render.call)
        self.assertIn('fake-output', self.render.call)
        self.assertNotIn('help', self.render.call)

    def test_magic_recipe_help(self):
        '''Should expose cell contents with a cell magic prefix as a help recipe.'''
        import mywb.test_injectable as mod
        self.assertTrue(callable(mod.r_recipe))
        self.assertFalse(hasattr(mod.r_recipe, 'help'))
        self.assertEqual(mod.r_recipe.__doc__, loader.UNBOUND_HELP_DOCSTRING_TMPL.format(name='r_recipe'))
        mod.r_recipe()
        self.assertIn('X', self.render.call)
        self.assertIn('1,2,3', self.render.call)
        self.assertIn(r'%%', self.render.call)
        self.assertNotIn('help', self.render.call)



# Copyright (c) Jupyter Development Team.
# Distributed under the terms of the Modified BSD License.

import unittest
import jupyter_cms.loader as loader
import jupyter_cms.inject as inject
from os.path import join as pjoin
from os.path import abspath, dirname

JS_FOR_HELP = '''var i = IPython.notebook.get_selected_index();
var cell = IPython.notebook.insert_cell_below('markdown', i+0);
cell.set_text("# A Recipe!");
cell.rendered = false; cell.render();
var cell = IPython.notebook.insert_cell_below('code', i+1);
cell.set_text("print x + y");
var self = this; setTimeout(function() { self.clear_output(); }, 0);'''

JS_FOR_API = '''var i = IPython.notebook.get_selected_index();
var cell = IPython.notebook.insert_cell_below('markdown', i+0);
cell.set_text("# A function!");
cell.rendered = false; cell.render();
var cell = IPython.notebook.insert_cell_below('code', i+1);
cell.set_text("print some_func()");
var self = this; setTimeout(function() { self.clear_output(); }, 0);'''

def spy():
    '''Really lightweight spy for tracking a single function call.'''
    def spy(value):
        assert spy.call is None
        spy.call = value
    spy.call = None
    return spy

class TestInjectMagic(unittest.TestCase):
    def setUp(self):
        loader.enable(pjoin(dirname(abspath(__file__)), 'resources'))

        self.magics = inject.InjectMagic()
        self.magics.shell = spy()

        # purpose driven mocking
        inject.display = lambda x: None
        self.render = inject.Javascript = spy()

    def tearDown(self):
        loader.disable()

    def test_inject_help(self):
        import mywb.test_injectable as mod

        self.magics.shell.user_module = mod
        self.magics.inject('some_recipe')

        self.assertEqual(self.render.call.strip(), JS_FOR_HELP)

    def test_inject_api(self):
        import mywb.test_injectable as mod

        self.magics.shell.user_module = mod
        self.magics.inject('some_func')

        self.assertEqual(self.render.call.strip(), JS_FOR_API)

    def test_inject_bad_path(self):
        import mywb.test_injectable as mod

        self.magics.shell.user_module = mod
        rv = self.magics.inject('a.b.some_func')

        assert 'Could not locate a.b.some_func' == rv
        assert self.render.call is None

    def test_inject_not_exposed(self):
        import mywb.test_injectable as mod

        self.magics.shell.user_module = mod
        rv = self.magics.inject('not_exposed')

        assert 'Could not locate not_exposed' == rv
        assert self.render.call is None

    def test_inject_unsupported(self):
        import mywb.test_injectable as mod

        self.magics.shell.user_module = mod
        rv = self.magics.inject('another_func')

        assert 'Injection of the requested object is not yet supported' == rv
        assert self.render.call is None